import 'package:iversioning/data/web/iversioning_json.dart';
import 'package:nui_core/model/interfaces.dart';
import 'package:nui_web/nui_web.dart';

abstract class _Apis{
  Future<DataResult<NUIAppVersionResponse>> getSupportedAppVersions(String version, String platform); //Get supported app versions
  Future<DataResult<NUIAppVersionActionResponse>> checkAppVersion(String version, String platform); //Get supported app versions
}

class IVersioningApi implements _Apis{
  static const String MODULE = "IVersioningApi";

  NUIWeb _getWeb() => NUIWeb.get(module: MODULE);

  void _processResponseMessage(NUIStandardHttpResponse response){
    if(response?.data?.containsKey("message") == true){
      response.message = response.data["message"];
    }
  }

  @override
  Future<DataResult<NUIAppVersionResponse>> getSupportedAppVersions(String version, String platform) async{
    final result = await _getWeb().doGET(url: "app-versions", entity: NUIStandardHttpResponseEntity());
    _processResponseMessage(result.body);
    if(result.body.data != null){
      final appVersions = NUIAppVersionResponseEnt().toEntity(result.body.data);
      return DataResult(success: result.success, message: result.message, data: appVersions);
    }
    return DataResult(success: result.success, message: result.message);
  }

  @override
  Future<DataResult<NUIAppVersionActionResponse>> checkAppVersion(String version, String platform) async{
    final result = await _getWeb().doGET(url: "app-versions/check", entity: NUIStandardHttpResponseEntity());
    _processResponseMessage(result.body);
    if(result.body.data != null){
      final appVersions = NUIAppVersionActionResponseEnt().toEntity(result.body.data);
      return DataResult(success: result.success, message: result.message, data: appVersions);
    }
    return DataResult(success: result.success, message: result.message);
  }

}